// Determines the colour assignments for each vertex from a HC for the converted instance.
//
// The input should be the original COL file, and a .hcp.tou file, such as the following:
//
// NAME : filename.hcp.tou
// COMMENT : HC for filename.hcp
// TYPE : TOUR
// DIMENSION : 1200
// TOUR_SECTION
// 1
// 2
// 3
// ...
// 1200
// -1
// EOF
//
//
// There is no need to input the number of colours as COL2HCPSol can work it out from the size of the HCP instance.
//
// Written by Michael Haythorpe

import java.io.*;
import java.util.StringTokenizer;

public class COL2HCPSol
{
   public static void main(String [] args)
   {
      if(args.length != 2)
      {
         System.out.println("ERROR: Input must be of the following form: COL2HCPSol <input.col> <output.hcp.tou>");
         return;
      }
   
      String inputFilename = args[0];
      String outputFilename = args[1];
      BufferedReader br;
      
      int N = -1;
      int edges = -1;
      int colours = -1;
   
      try
      {
         br = new BufferedReader(new FileReader(inputFilename));
      }
      catch(Exception e)
      {
         System.out.println("ERROR: Cannot open file " + inputFilename);
         return;
      }
   
      String line = "";
   
      try
      {
         line = br.readLine();
         while(line.charAt(0) != 'p')
         {
            line = br.readLine();
            if(line == null)
            {
               System.out.println("ERROR: " + inputFilename + " is not a valid COL file!");
               br.close();
               return;
            }
         }
         
         StringTokenizer tokens = new StringTokenizer(line);
         tokens.nextToken();
         tokens.nextToken();
         
         N = Integer.parseInt(tokens.nextToken());
         edges = Integer.parseInt(tokens.nextToken())/2;
         br.close();
               
      }
      catch(Exception e)
      {
         System.out.println("ERROR: " + inputFilename + " is not a valid CNF file!" + line);
         return;
      }
   
   
      try
      {
         br = new BufferedReader(new FileReader(outputFilename));
      }
      catch(Exception e)
      {
         System.out.println("ERROR: Cannot open file " + outputFilename);
         return;
      }
         
      line = "";
   		
      int colourset[] = new int[N];
      try
      {
         line = br.readLine();
         line = br.readLine();
         line = br.readLine();
         line = br.readLine();
         StringTokenizer tokens = new StringTokenizer(line);
         tokens.nextToken();
         tokens.nextToken();
         int nodes = Integer.parseInt(tokens.nextToken());
         colours = (nodes - 6*edges - 6)/(6*N + 18*edges);
         line = br.readLine();
         line = br.readLine();
         int []HC = new int[nodes];
         for(int i=0; i<nodes; i++)
         {
            HC[i] = Integer.parseInt(line);
            line = br.readLine();   
         }
         
         
         if(HC[1] != 2)
         {
            if(HC[nodes-1] != 2)
            {
               System.out.println("Not a valid HC!");
               return;
            }
            int HC2[] = new int[nodes];
            HC2[0] = 1;
            for(int i=1; i<nodes; i++)
               HC2[i] = HC[nodes-i];
            HC = HC2;
         }
      
      
         int count = 0;
         for(int k=0; k<colours; k++)
            for(int i=0; i<N; i++)
            {
               while(HC[count] != 6 + 6*k*N + 6*i)
                  count++;
            
               if(HC[count+1] <= 3+6*colours*N)
                  colourset[i] = k+1;
            
            
            }
            
         for(int i=0; i<N; i++)
            System.out.println("Vertex " + (i+1) + " colour : " + colourset[i]);
            
         br.close();
      
      
      }
      catch(Exception e)
      {
         System.out.println(e);
      }
   }
}